﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.ComponentModel;
using Infragistics.Controls.Menus;

namespace TestTreeView
{
    public class TestClass : INotifyPropertyChanged
    {
        #region Properties
        private string _name;
        public string Name
        {
            get { return _name; }
            set
            {
                if (_name != value)
                {
                    _name = value;
                    Changed("Name");
                }
            }
        }

        private bool _isExpanded;
        public bool IsExpanded
        {
            get { return _isExpanded; }
            set
            {
                if (_isExpanded != value)
                {
                    _isExpanded = value;
                    Changed("IsExpanded");
                }
            }
        }

        private XamDataTreeNode _xamNode;
        public XamDataTreeNode XamNode
        {
            get { return _xamNode; }
            set
            {
                if (_xamNode != value)
                {
                    _xamNode = value;

                    if (_xamNode != null)
                    {
                        if (XamNode.IsSelected != this.IsSelected)
                        {
                            XamNode.IsActive = this.IsSelected;
                            XamNode.IsSelected = this.IsSelected;
                        }

                        _xamNode.PropertyChanged += new PropertyChangedEventHandler(_xamNode_PropertyChanged);
                    } 
                }
            }
        }
        void _xamNode_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            switch (e.PropertyName)
            {
                case "IsSelected":
                    IsSelected = _xamNode.IsSelected;
                    break;

                default:
                    break;
            }
        }

        private bool _isSelected;
        public bool IsSelected
        {
            get { return _isSelected; }
            set
            {
                if (_isSelected != value)
                {
                    _isSelected = value;
                    Changed("IsSelected");

                    if (XamNode != null && XamNode.IsSelected != value)
                    {
                        XamNode.IsActive = value;
                        XamNode.IsSelected = value;
                    }
                }
            }
        }

        public ObservableCollection<TestClass> Children { get; set; } 
        #endregion
        
        #region Constructors
        public TestClass()
        {
            Children = new ObservableCollection<TestClass>();
            Name = "NA";
            IsExpanded = false;
        }
        public TestClass(string name)
            : this()
        {
            Name = name;
        }
        public TestClass(int level, int count)
            : this()
        {
            Name = string.Format("Node {0}-{1}", level, count);

            if (level > 0)
            {
                for (int i = 0; i < level; i++)
                {
                    Children.Add(new TestClass(level - 1, i+1));
                }
            }
        } 
        #endregion

        #region Methods
        public void SelectNode(List<string> nodePathList)
        {
            if (nodePathList.Count > 0)
            {
                var targetName = nodePathList[0];
                nodePathList.RemoveAt(0);

                // Find the target node
                var qNode = from n in Children where n.Name == targetName select n;
                if (qNode.Count() > 0)
                {
                    this.IsExpanded = true;

                    var node = qNode.First();

                    node.SelectNode(nodePathList);
                }
            }
            else
            {
                this.IsSelected = true;
            }
        }
        #endregion


        #region INotifyPropertyChanged Members

        private void Changed(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion
    }
}
